/* simplistic.y
 * A simplistic English grammar.
 */

%glr-parser

%{
  #include "shared.h"
  #define YYSTYPE shared
  #define YYDEBUG 1

  extern char *yytext;
  int yydebug, yyprintmerges;
  static void yyerror (char*);
  static shared merge (shared, shared);
%}


%token N V P D R C

%%

s:
    np vp                 { $$ = shared_rule("s", 2, $1, $2);
                            print_rule($$, 2, $1, $2); }
  | s pp                  { $$ = shared_rule("s", 2, $1, $2);
                            print_rule($$, 2, $1, $2); }
  ;

np:
    N                     { $$ = shared_rule("np", 1, $1);
                            print_rule($$, 1, $1); }
  | D N                   { $$ = shared_rule("np", 2, $1, $2);
                            print_rule($$, 2, $1, $2); }
  | np pp                 { $$ = shared_rule("np", 2, $1, $2);
                            print_rule($$, 2, $1, $2); }
  ;

pp:
    P np                  { $$ = shared_rule("pp", 2, $1, $2);
                            print_rule($$, 2, $1, $2); }
  ;

vp:
    V                     { $$ = shared_rule("vp", 1, $1);
                            print_rule($$, 1, $1); }
  | V np                  { $$ = shared_rule("vp", 2, $1, $2);
                            print_rule($$, 2, $1, $2); }
  | V rel s               { $$ = shared_rule("vp", 3, $1, $2, $3);
                            print_rule($$, 3, $1, $2, $3); }
  ;

rel:
    R                     { $$ = shared_rule("rel", 1, $1);
                            print_rule($$, 1, $1); }
  | /* empty */           { $$ = shared_rule("rel", 0);
                            print_rule($$, 0); }
  ;

%%
static void
yyerror (char* msg)
{
  fprintf(stderr, "Error: %s before \"%s\".\n", msg, yytext);
}

static shared
merge (shared s0, shared s1)
{
  if (yyprintmerges)
    {
      fprintf (stderr, "# Merged ");
      print_shared (stderr, s0);
      fprintf (stderr, " and ");
      print_shared (stderr, s1);
      fprintf (stderr, ".\n");
    }
  return s1;
}

int
main (int argc, char *argv[])
{
  int ret = 0;
  yytokenpos = yydebug = yyprintmerges = 0;

  if (argc == 2 && !strcmp(argv[1],"--trace"))
    yydebug = 1;
  if (argc == 2 && !strcmp(argv[1],"--merge"))
    yyprintmerges = 1;
  if (argc == 2 && !strcmp(argv[1],"--dot"))
    {
      is_dot_format = TRUE;
      printf ("digraph {\n");
    }

  if ((ret = yyparse()))
    fprintf (stderr, "Parsing failed.\n");

  if (is_dot_format)
    printf ("}\n");

  return ret;
}
